@@ -5,7 +5,7 @@ use strict;
 use warnings;
 
 package Mail::IMAPClient;
-our $VERSION = '3.18';
+our $VERSION = '3.19';
 
 use Mail::IMAPClient::MessageSet;
 
@@ -450,7 +450,16 @@ sub _list_or_lsub {
     $self->_imap_command(qq($cmd "$reference" $target))
       or return undef;
 
-    return wantarray ? $self->History : $self->Results;
+    # cleanup any literal data that may be returned
+    my $ret = wantarray ? [ $self->History ] : $self->Results;
+    if ($ret) {
+        my $cmd = wantarray ? undef : shift @$ret;
+        $self->_list_response_preprocess($ret);
+        unshift( @$ret, $cmd ) if defined($cmd);
+    }
+
+    #return wantarray ? $self->History : $self->Results;
+    return wantarray ? @$ret : $ret;
 }
 
 sub list { shift->_list_or_lsub( "LIST", @_ ) }
@@ -487,8 +496,6 @@ sub _folders_or_subscribed {
                 push @list, @$tref;
             }
 
-            $self->_list_response_preprocess( \@list );    # necessary? remove?
-
             foreach my $resp (@list) {
                 my $rec = $self->_list_or_lsub_response_parse($resp);
                 next unless defined $rec->{name};
@@ -508,14 +515,14 @@ sub folders {
       if !$what && $self->{Folders};
 
     my @folders = $self->_folders_or_subscribed( "list", $what );
-
     $self->{Folders} = \@folders unless $what;
     return wantarray ? @folders : \@folders;
 }
 
 sub subscribed {
     my ( $self, $what ) = @_;
-    $self->_folders_or_subscribed( "lsub", $what );
+    my @folders = $self->_folders_or_subscribed( "lsub", $what );
+    return wantarray ? @folders : \@folders;
 }
 
 # BUG? cleanup escaping/quoting
@@ -1368,28 +1375,25 @@ sub _record {
     push @{ $self->{History}{$count} }, $array;
 }
 
-#_send_line writes to the socket:
+# _send_line handles literal data and supports the Prewritemethod
 sub _send_line {
     my ( $self, $string, $suppress ) = ( shift, shift, shift );
 
-    $string =~ s/\r?\n?$/$CRLF/o
+    $string =~ s/$CR?$LF?$/$CRLF/o
       unless $suppress;
 
-    if ( $string =~ s/^([^\n{]*\{(\d+)\}$CRLF)(.)/$3/o ) {
-
-        # string starts with literal
-        my ( $first, $len ) = ( $1, $2 );
+    # handle case where string contains a literal
+    if ( $string =~ s/^([^$LF\{]*\{\d+\}$CRLF)(?=.)//o ) {
+        my $first = $1;
         $self->_debug("Sending literal: $first\tthen: $string");
-
         $self->_send_line($first) or return undef;
 
         # look for "<anything> OK|NO|BAD" or "+..."
         my $code = $self->_get_response( qr(\S+), '+' ) or return undef;
         return undef unless $code eq '+';
-
-        # non-literal part continues below...
     }
 
+    # non-literal part continues...
     unless ( $self->IsConnected ) {
         $self->LastError("NO not connected");
         return undef;
@@ -1650,7 +1654,7 @@ sub _read_line {
         }
     }
 
-    $self->_debug( "Read: " . join "\n      ", map { $_->[DATA] } @$oBuffer );
+    $self->_debug( "Read: " . join "", map { "\t" . $_->[DATA] } @$oBuffer );
     @$oBuffer ? $oBuffer : undef;
 }
 
@@ -1758,6 +1762,8 @@ sub _disconnect {
 # LIST or LSUB Response
 #   Contents: name attributes, hierarchy delimiter, name
 #   Example: * LIST (\Noselect) "/" ~/Mail/foo
+# NOTE: in _list_response_preprocess we append literal data so we need
+# to be liberal about our matching of folder name data
 sub _list_or_lsub_response_parse {
     my ( $self, $resp ) = @_;
 
@@ -1766,10 +1772,10 @@ sub _list_or_lsub_response_parse {
 
     $resp =~ s/\015?\012$//;
     if (
-        $resp =~ / ^\* \s+ (?:LIST|LSUB) \s+
-              \( ([^\)]*) \)          \s+   # * LIST (attrs)
-           (?:\" ([^"]*)  \" | NIL  ) \s+   # "delimiter" or NIL
-           (?:\" (.*)     \" | (\S+))       # "name" or name
+        $resp =~ / ^\* \s+ (?:LIST|LSUB) \s+   # * LIST or LSUB
+                 \( ([^\)]*) \)          \s+   # (attrs)
+           (?:   \" ([^"]*)  \" | NIL  ) \s    # "delimiter" or NIL
+           (?:\s*\" (.*)     \" | (.*) )       # "name" or name
          /ix
       )
     {
@@ -1779,18 +1785,19 @@ sub _list_or_lsub_response_parse {
     return wantarray ? %info : \%info;
 }
 
-# BUG? Legacy code does this for subscribed() and folders(). Is there
-# a case where lines returned by the server do not end in $CRLF and we
-# need to treat them as a continuation of the previous line?
+# handle listeral data returned in list/lsub responses
+# some example responses:
+# * LIST () "/" "My Folder"    # nothing to do here...
+# * LIST () "/" {9}            # the {9} is already removed by _read_line()
+# Special %                    # we append this to the previous line
 sub _list_response_preprocess {
     my ( $self, $data ) = @_;
     return undef unless defined $data;
 
     for ( my $m = 0 ; $m < @$data ; $m++ ) {
-        if ( $data->[$m] && $data->[$m] !~ /$CRLF$/o ) {
-            local ($!);    # old versions of Carp could reset $!
-            carp("concatenating $data->[$m] and $data->[$m+1]");
-            $data->[$m] .= $data->[ $m + 1 ];
+        if ( $data->[$m] && $data->[$m] !~ /$CR?$LF$/o ) {
+            $self->_debug("concatenating '$data->[$m]' and '$data->[$m+1]'");
+            $data->[$m] .= " " . $data->[ $m + 1 ];
             splice @$data, $m + 1, 1;
         }
     }
@@ -2231,6 +2238,11 @@ sub parse_headers {
     my %fieldmap = map { ( lc($_) => $_ ) } @fields;
     my $msgid;
 
+    # some example responses:
+    # * OK Message 1 no longer exists
+    # * 1 FETCH (UID 26535 BODY[HEADER] "")
+    # * 5 FETCH (UID 30699 BODY[HEADER] {1711}
+    # header: value...
     foreach my $header ( map { split /$CR?$LF/o } @$raw ) {
 
         # little problem: Windows2003 has UID as body, not in header
@@ -2238,7 +2250,7 @@ sub parse_headers {
             $header =~ s/^\* \s+ (\d+) \s+ FETCH \s+
                         \( (.*?) BODY\[HEADER (?:\.FIELDS)? .*? \]\s*//ix
           )
-        {           # start new message header
+        {    # start new message header
             ( $msgid, my $msgattrs ) = ( $1, $2 );
             $h = {};
             if ( $self->Uid )    # undef when win2003
@@ -2262,7 +2274,6 @@ sub parse_headers {
         }
 
         unless ( defined $h ) {
-            last if $header =~ / OK /i;
             $self->_debug("found data between fetch headers: $header");
             next;
         }
@@ -2274,6 +2285,12 @@ sub parse_headers {
         elsif ( $field and ref $h->{$field} eq 'ARRAY' ) {    # folded header
             $h->{$field}[-1] .= $header;
         }
+        else {
+
+            # show data if it is not like  '"")' or '{123}'
+            $self->_debug("non-header data between fetch headers: $header")
+              if ( $header !~ /^(?:\s*\"\"\)|\{\d+\})$CR?$LF$/o );
+        }
     }
 
     # if we asked for one message, just return its hash,
@@ -2402,6 +2419,9 @@ sub _quote_search {
         elsif ( exists $SEARCH_KEYS{ uc($_) } ) {
             push( @ret, $v );
         }
+        elsif ( @args == 1 ) {
+            push( @ret, $v );    # <3.17 compat: caller responsible for quoting
+        }
         else {
             push( @ret, $self->Quote($v) );
         }
@@ -2410,11 +2430,11 @@ sub _quote_search {
 }
 
 sub search {
-    my ( $self, @a ) = @_;
+    my ( $self, @args ) = @_;
 
-    @a = $self->_quote_search(@a);
+    @args = $self->_quote_search(@args);
 
-    $self->_imap_uid_command( SEARCH => @a )
+    $self->_imap_uid_command( SEARCH => @args )
       or return undef;
 
     my @hits;
@@ -2619,9 +2639,6 @@ sub is_parent {
     my ( $self, $folder ) = ( shift, shift );
     my $list = $self->list( undef, $folder ) or return undef;
 
-    shift @$list;                               # remove command
-    $self->_list_response_preprocess($list);    # necessary? remove?
-
     my $attrs;
     foreach my $resp (@$list) {
         my $rec = $self->_list_or_lsub_response_parse($resp);
